 /*****************************************************************************
 * CDD v1.0 Copyright (C) 1996, Charles R. Henrich, All Rights Reserved.      *
 *                                                                            *
 * Use in source and binary forms, with or without modification and           *
 * Redistribution without modification are permitted provided that the        *
 * following conditions are met:                                              *
 *                                                                            *
 * 1. Redistributions of source code must retain the above copyright          *
 *    notice, all internal copyright notices, this list of conditions         *
 *    and the following disclaimer.                                           *
 * 2. Redistributions in binary form must reproduce the above copyright       *
 *    notice, this list of conditions and the following disclaimer in the     *
 *    documentation and/or other materials provided with the distribution.    *
 * 3. All advertising materials mentioning features or use of this software   *
 *    must display the following acknowledgement:                             *
 *      CDD was developed in 1996 by Charles Henrich                          *
 * 4. Neither the name of this product (CDD) nor the name of the author       *
 *    (Charles Henrich) may be used to endorse products that include this     *
 *    product without specific prior written permission.                      *
 * 5. No profit may be made from the inclusion, or distribution of this       *
 *    product.  Although a reasonable service charge for the distribution     *
 *    of this product is permitted where appropriate.                         *
 *                                                                            *
 * THIS SOFTWARE IS PROVIDED BY CHARLES HENRICH `` AS IS '' AND ANY EXPRESS   *
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED          *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE     *
 * DISCLAIMED.  IN NO EVENT SHALL CHARLES HENRICH BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES         *
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR         *
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)         *
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT *
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY  *
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF     *
 * SUCH DAMAGE.                                                               *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Email: henrich@msu.edu                                                     *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <time.h>
#ifdef __NetBSD__
#include <sys/ioctl.h>
#include <sys/audioio.h>
#else
#include <machine/soundcard.h>
#endif
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <unistd.h>
#include <fcntl.h>
#include <scsi.h>
#include "cdd.h"
#include "cdd_cdcmds.h"
#include "cdd_util.h"
#ifdef NETWORK_DB
#include "cdd_cddb.h"
#endif

OPTS G_opts;

int F_cdctlfd;

int main(int argc, char *argv[])
{
CDTOC cdtoc;
int count;

strcpy(G_opts.workdir, "/tmp");
G_opts.onlyaudio = FALSE;
G_opts.onlydata = FALSE;
G_opts.track = -1;
G_opts.rawmode = FALSE;
G_opts.rawmodecorrect = TRUE;
G_opts.bequiet = FALSE;
G_opts.dostdout = FALSE;
G_opts.cddb = FALSE;

signal(SIGINT, resetcd);
signal(SIGBUS, resetcd);
signal(SIGSEGV, resetcd);

#ifdef __NetBSD__
F_cdctlfd=scsi_open("/dev/rcd0d", O_RDWR);
#else
F_cdctlfd=scsi_open("/dev/rcd0.ctl", O_RDWR);
#endif

if(F_cdctlfd < 0)
    {
#ifdef __NetBSD__
    fprintf(stderr,"Unable to open /dev/rcd0d [%d]\n", errno);
#else
    fprintf(stderr,"Unable to open /dev/rcd0.ctl [%d]\n", errno);
#endif
    exit(0);
    }

for(count=1;count<argc;count++)
    {
    if(argv[count][0] != '-') strcpy(G_opts.workdir, argv[count]);

    if(strncasecmp(argv[count],"-", 2) == 0) 
        {
        G_opts.dostdout = TRUE;
        G_opts.bequiet = TRUE;
        }

    if(strncasecmp(argv[count],"-a", 2) == 0) G_opts.onlyaudio = TRUE;
    if(strncasecmp(argv[count],"-d", 2) == 0) G_opts.onlydata = TRUE;
    if(strncasecmp(argv[count],"-r", 2) == 0) G_opts.rawmode = TRUE;
    if(strncasecmp(argv[count],"-p", 2) == 0) G_opts.playaudio = TRUE;
    if(strncasecmp(argv[count],"-q", 2) == 0) G_opts.bequiet = TRUE;
    if(strncasecmp(argv[count],"-n", 2) == 0) G_opts.cddb = TRUE;
    if(strncasecmp(argv[count],"-noed", 5) == 0) G_opts.rawmodecorrect = FALSE;
    if(strncasecmp(argv[count],"-t", 2) == 0) 
        {
        if(count+1 == argc)
            {
            fprintf(stderr,"Copy what track ?\n");
            close(F_cdctlfd);
            exit(0);
            }

        G_opts.track = atoi(argv[count+1]);
        count++;
        }

    if(strncasecmp(argv[count],"-h", 2) == 0) 
        {
        fprintf(stderr,"\nCDD v%s, Copyright (C) 1996, ", VERSION);
        fprintf(stderr,"Charles R. Henrich, All Rights Reserved.\n\n");

        fprintf(stderr,"Usage: %s [-t #][-a][-d][-r][-n][-noed][-h]", argv[0]);
        fprintf(stderr,"[-p][-q][destdir|-]\n\n");
        fprintf(stderr,"    -h      This help\n");
        fprintf(stderr,"    -t #    Copy only Track #\n");
        fprintf(stderr,"    -a      Copy only AUDIO tracks\n");
        fprintf(stderr,"    -p      Play AUDIO during Copy\n");
        fprintf(stderr,"    -d      Copy only DATA tracks\n");
        fprintf(stderr,"    -q      Run silently\n");
        fprintf(stderr,"    -n      Lookup track names from network\n");
        fprintf(stderr,"    -       Write data on stdout\n");
        fprintf(stderr,"    -r      Copy DATA tracks in RAW mode (includes");
        fprintf(stderr," Sync+Header and EDC/ECC bits\n");
        fprintf(stderr,"    -noed   Copy RAW DATA tracks without performing ");
        fprintf(stderr,"any error checking\n\n");
        fprintf(stderr,"Running CDD with no options will show the table of ");
        fprintf(stderr,"contents of the CD in the \ndrive\n\n");
        close(F_cdctlfd);
        exit(0);
        }
    }

cdd_printf("\nCDD v%s, Copyright (C) 1996, ", VERSION);
cdd_printf("Charles R. Henrich, All Rights Reserved.\n\n");

doinquiry(F_cdctlfd);

if(getcdtoc(F_cdctlfd, &cdtoc) != SUCCESS)
    {
    fprintf(stderr,"Unable to read TOC\n");
    close(F_cdctlfd);
    exit(0);
    }

cdtoc.cdname[0] = '\0';

for(count=0;count<cdtoc.numentries;count++) 
    {
    cdtoc.tocents[count].trackname[0] = '\0';
    }

#ifdef NETWORK_DB
if(G_opts.cddb == TRUE && getcddbtoc(&cdtoc) != SUCCESS)
    {
    cdd_printf("Unable to retrieve CD-DB entries from network\n\n");
    }
else
    {
    cdd_printf("CD: %s\n\n", cdtoc.cdname);
    }

#endif

showcdtoc(&cdtoc);

if(G_opts.onlyaudio == TRUE && G_opts.onlydata == TRUE)
    {
    fprintf(stderr,"Only audio OR only data, not both!\n");
    close(F_cdctlfd);
    exit(0);
    }

if(argc == 2 && G_opts.cddb == TRUE)
    {
    close(F_cdctlfd);
    exit(0);
    }

if(argc > 1)
    {
    if(G_opts.onlyaudio == FALSE && G_opts.onlydata == FALSE && 
       G_opts.track == -1)
        {
        cdd_printf("Copying all tracks\n");
        }

    if(G_opts.onlyaudio == TRUE) cdd_printf("Copying only AUDIO tracks\n");
    if(G_opts.onlydata == TRUE) cdd_printf("Copying only DATA tracks\n");
    if(G_opts.track != -1) cdd_printf("Copying only track %d\n", G_opts.track);
    if(G_opts.rawmode == TRUE) 
        {
        cdd_printf("Copying DATA tracks in RAW mode ");
        
        if(G_opts.rawmodecorrect == TRUE) cdd_printf("WITH error detection\n");
        else                           cdd_printf("WITHOUT error detection\n");
        }
    }

if(argc > 1) copycdtoc(&cdtoc, &G_opts);

close(F_cdctlfd);

if(argc > 1) cdd_printf("Copy Complete\n");

exit(0);
}

void resetcd(int signo)
{
cd_setblocksize(F_cdctlfd, 2048);
close(F_cdctlfd);
exit(0);
}

void showcdtoc(CDTOC *cdtoc)
{
int count;

cdd_printf("%d Tracks: %d - %d\n\n", 
       cdtoc->numentries - 1, cdtoc->firsttrack, cdtoc->lasttrack);
cdd_printf("                       Pre        Copy\n");
cdd_printf("Track#   Type        Emphasis   Prohibit   ");
cdd_printf("Channels   Start (M:S:F/LBA)  Length\n");

for(count=0;count<cdtoc->numentries;count++)
    {
    if(cdtoc->tocents[count].tracknumber >= 0xA0) 
        {
        cdd_printf("  LO     ");
        }
    else              
        {
        cdd_printf("  %2.2d     ",cdtoc->tocents[count].tracknumber);
        }

    PRINT(cdtoc->tocents[count].ttype,   "%-8.8s      ", "CD-AUDIO", "CD-ROM");
    PRINT(cdtoc->tocents[count].preemph, "%-3.3s        ", "NO", "YES");
    PRINT(cdtoc->tocents[count].dcp,     "%-3.3s        ", "YES", "NO");
    PRINT(cdtoc->tocents[count].nchan, "%-4.4s     ", "TWO", "FOUR");
    printmsf(cdtoc->tocents[count].absaddr);
    cdd_printf(" / %6.6u  ", cdtoc->tocents[count].absaddr);

    if(cdtoc->tocents[count].tracknumber < 0xA0)
        {
        cdd_printf("%u\n", 
               cdtoc->tocents[count].absendaddr-cdtoc->tocents[count].absaddr); 
        }
    else 
        {
        cdd_printf("------\n");
        }

    if(cdtoc->tocents[count].trackname[0] != '\0') 
        {
        cdd_printf("\t(%s)\n", cdtoc->tocents[count].trackname);
        }
    }

cdd_printf("\n");
}

void copycdtoc(CDTOC *cdtoc, OPTS *opts)
{
int count;
char filename[MAXPATHLEN];

cdd_printf("\n");

if(G_opts.dostdout == FALSE) mkdir(opts->workdir, 0700);

for(count=0;count<cdtoc->numentries;count++)
    {
    if(cdtoc->tocents[count].tracknumber >= 0xA0) break;

    if(opts->track != -1 && 
       cdtoc->tocents[count].tracknumber != opts->track) continue;

    if(cdtoc->tocents[count].ttype == 0) 
        {
        if(opts->onlydata == TRUE) continue;

        if(cdtoc->tocents[count].trackname[0] != '\0')
            {
            sprintf(filename, "%s/%2.2d %s.cda", opts->workdir,
                    cdtoc->tocents[count].tracknumber,
                    cdtoc->tocents[count].trackname);
            }
        else
            {
            sprintf(filename, "%s/track-%2.2d.cda", 
                    opts->workdir, cdtoc->tocents[count].tracknumber);
            }

        cdd_printf("Copying AUDIO track %d to %s\n\n", 
               cdtoc->tocents[count].tracknumber, filename);

        copycdaudio_to_cda(&cdtoc->tocents[count], filename, opts);
        }
    else
        {
        if(opts->onlyaudio == TRUE) continue;

        if(opts->rawmode == TRUE)
            {
            sprintf(filename, "%s/track-%2.2d.raw", 
                    opts->workdir, cdtoc->tocents[count].tracknumber);

            cdd_printf("Copying RAW DATA track %d to %s\n\n", 
                   cdtoc->tocents[count].tracknumber, filename);
 
            copyrawcddata_to_file(&cdtoc->tocents[count], filename, opts);
            }
        else
            {
            sprintf(filename, "%s/track-%2.2d.dd", 
                    opts->workdir, cdtoc->tocents[count].tracknumber);

            cdd_printf("Copying DATA track %d to %s\n\n", 
                   cdtoc->tocents[count].tracknumber, filename);

            copycddata_to_file(&cdtoc->tocents[count], filename, opts);
            }
        }
    }
}

void copycddata_to_file(CDTOCENT *cdtocent, char *filename, OPTS *opts)
{
char buffer[CDDATAREADSIZE];
long starttme;
int ifd;
int ofd;
int count;
int ret;
int bytesread;

#ifdef __NetBSD__
ifd=dup(F_cdctlfd);
#else
ifd=open("/dev/rcd0c", O_RDONLY, 0600);
#endif

if(ifd < 0)
    {
#ifdef __NetBSD__
    fprintf(stderr,"Unable to dup descriptor\n");
#else
    fprintf(stderr,"Unable to open /dev/rcd0c\n");
#endif
    close(F_cdctlfd);
    exit(0);
    }

if(G_opts.dostdout == FALSE)
    {
    ofd=open(filename, O_CREAT|O_TRUNC|O_WRONLY, 0600);

    if(ofd < 0)
        {
        fprintf(stderr,"Unable to open %s\n", filename);
        close(ifd);
        close(F_cdctlfd);
        exit(0);
        }
    }
else
    {
    ofd = fileno(stdout);
    }

starttme = time(NULL);

for(count=0,bytesread=read(ifd, buffer, CDDATAREADSIZE);
    bytesread > 0;
    count+=bytesread, bytesread=read(ifd, buffer, CDDATAREADSIZE))
    {
    ret=write(ofd, buffer, bytesread);
    if(ret != bytesread)
        {
        fprintf(stderr,"Unable to write raw CD data\n");
        if(G_opts.dostdout == FALSE) close(ofd);
        close(ifd);
        close(F_cdctlfd);
        exit(0);
        }

    cdd_printf("MB: %.2f KB/sec: %.2f\r", count*1.0/1048576, 
           count*1.0/1024/(time(NULL)-starttme+1));

    fflush(stdout);
    }

close(ifd);

if(bytesread == -1 && count == 0)
    {
    fprintf(stderr,"Unable to read from CD, errno %d\n", errno);
    close(F_cdctlfd);
    exit(0);
    }

/* This nonsense is to get the CD-ROM to be sitting back on sector zero,
   and ready for further commands */

#ifdef __NetBSD__
ifd=dup(F_cdctlfd);
#else
ifd=open("/dev/rcd0c", O_RDONLY);
#endif

if(ifd < 0)
    {
#ifdef __NetBSD__
    fprintf(stderr,"Unable to dup descriptor\n");
#else
    fprintf(stderr,"Unable to open /dev/rcd0c\n");
#endif
    close(F_cdctlfd);
    exit(0);
    }

read(ifd, buffer, CDDATAREADSIZE);

close(ifd);

/* End Nonsense */

cdd_printf("\n");

if(G_opts.dostdout == FALSE) close(ofd);

return;
}

void copyrawcddata_to_file(CDTOCENT *cdtocent, char *filename, OPTS *opts)
{
unsigned char buffer[CDRAWREADSIZE];
char correctfile[BUFSIZ];
unsigned char correctbuf[CDDATAREADSIZE];
FILE *ifp = NULL;
int ofd;
int min;
int sec;
int frame;
int count;
int curlba;
int framesperread;
long starttme;
u_int absoluteframe;
u_int expectedframe;
u_int edc;

if(opts->rawmodecorrect == TRUE)
    {
    sprintf(correctfile, "%s.correct", filename);
    
    copycddata_to_file(cdtocent, correctfile, opts);
    
    cdd_printf("\n");
    
    ifp=fopen(correctfile, "r");
    
    if(ifp == NULL)
        {
        fprintf(stderr,"Unable to open correction file %s\n", correctfile);
        close(F_cdctlfd);
        exit(0);
        }

    if(remove(correctfile) != 0)
        {
        cdd_printf("Unable to remove correction file %s\n", correctfile);
        }
    }

if(G_opts.dostdout == FALSE)
    {
    ofd=open(filename, O_CREAT|O_TRUNC|O_WRONLY, 0600);

    if(ofd < 0)
        {
        fprintf(stderr,"Unable to open %s\n", filename);
        close(F_cdctlfd);
        exit(0);
        }
    }
else
    {
    ofd = fileno(stdout);
    }

cd_setblocksize(F_cdctlfd, CDRAWFRAMESIZE);

absoluteframe = expectedframe = cdtocent->absaddr + 150;

framesperread = FRAMESPERREAD;

starttme = time(NULL);

for(curlba = cdtocent->absaddr;
    curlba < cdtocent->absendaddr;
    curlba+=framesperread)
    {
    if(framesperread > cdtocent->absendaddr - curlba) 
        {
        framesperread = cdtocent->absendaddr - curlba;
        cdd_printf("\n\nAdjusted framesperread to %d (%d:%d)\n\n", 
                framesperread, cdtocent->absendaddr, curlba);
        }
    
    if(cdrawread(buffer, curlba, CDRAWREADSIZE, framesperread, F_cdctlfd) != 
       SUCCESS)
        {
        cdd_printf("\nUnable to rawread at lba %d\n",curlba);
        break;
        }

    if(opts->rawmodecorrect == TRUE)
        {
        if(fread(correctbuf, CDDATAFRAMESIZE, framesperread, ifp) != 
           framesperread)
            {
            fprintf(stderr,"\nCant read correction buffer!\n");
            if(G_opts.dostdout == FALSE) close(ofd);
            fclose(ifp);
            cd_setblocksize(F_cdctlfd, 2048);
            close(F_cdctlfd);
            }
        }

    for(count=0;count < framesperread; count++, expectedframe++)
        {
        min = bcdtodec(&buffer[(count * CDRAWFRAMESIZE) + 12]);
        sec = bcdtodec(&buffer[(count * CDRAWFRAMESIZE) + 13]);
        frame = bcdtodec(&buffer[(count * CDRAWFRAMESIZE) + 14]);

        if(buffer[15] == 2) /* CD-ROM Mode 2 */
            {
            if(opts->rawmodecorrect == TRUE)
                {
                if(memcmp(&correctbuf[CDDATAFRAMESIZE*count],
                          &buffer[CDRAWFRAMESIZE*count+24], 
                          CDDATAFRAMESIZE) != 0)
                    {
                    fprintf(stderr, "\nFrame (%x:%x) %d INVALID!!\n", 
                            CDDATAFRAMESIZE*count, 
                            CDRAWFRAMESIZE*count+24, curlba+count);
                    }
                }

            memcpy(&edc, &buffer[(count * CDRAWFRAMESIZE) + 2072], 4);
            }
        else                /* CD-ROM Mode 1 */
            {
            if(opts->rawmodecorrect == TRUE)
                {
                if(memcmp(&correctbuf[CDDATAFRAMESIZE*count],
                          &buffer[CDRAWFRAMESIZE*count+16], 
                          CDDATAFRAMESIZE) != 0)
                    {
                    fprintf(stderr,"\nFrame (%x:%x) %d INVALID!!\n", 
                            CDDATAFRAMESIZE*count, 
                            CDRAWFRAMESIZE*count+16, curlba+count);
                    }
                }

            memcpy(&edc, &buffer[(count * CDRAWFRAMESIZE) + 2064], 4);
            }
    
        absoluteframe = (min * 60 * 75) + (sec * 75) + frame;

        if(absoluteframe != expectedframe)
            {
            fprintf(stderr,"\n\nUnable to read frame %d, aborting..\n",
                    expectedframe);
    
            if(G_opts.dostdout == FALSE) close(ofd);
            if(opts->rawmodecorrect == TRUE) fclose(ifp);
            cd_setblocksize(F_cdctlfd, 2048);
            close(F_cdctlfd);
            exit(0);
            }

    /* Absolute frame is already 150 frames away from absendaddr, however due
       to the pending mode change (ever hear of a multitrack ROM disc?) we want
       to make sure we back off another 300, as to not lock up the system.
       In any case as we're only pulling back when EDC == 0 (invalid frame) it
       shouldnt make shit for difference, and we dont really want the EDC 0
       frame anyway. */ 

        if(edc == 0 && absoluteframe+150 >= cdtocent->absendaddr)
            {
            cdd_printf("\n\nBreaking with zero EDC and ");
            cdd_printf("close to EOF (%d:%d)\n",
                    absoluteframe, cdtocent->absendaddr);

            goto done;
            break;
            }

        if(write(ofd, &buffer[count*CDRAWFRAMESIZE], 
           CDRAWFRAMESIZE) != CDRAWFRAMESIZE)
            {
            fprintf(stderr,"Unable to write raw CD data\n");
            if(opts->rawmodecorrect == TRUE) fclose(ifp);
            if(G_opts.dostdout == FALSE) close(ofd);
            cd_setblocksize(F_cdctlfd, 2048);
            close(F_cdctlfd);
            exit(0);
            }
        }

    cdd_printf("CLBA: %d EDC: %8.8x KB/sec: %.2f\r", curlba+count, edc,
           (curlba+count-1-cdtocent->absaddr)*2.0/(time(NULL)-starttme+1));

    fflush(stdout);
    }

done: 

cdd_printf("\n\nLastframe read %d\n\n", absoluteframe-150);
if(opts->rawmodecorrect == TRUE) fclose(ifp);
if(G_opts.dostdout == FALSE) close(ofd);
cd_setblocksize(F_cdctlfd, 2048);
}

void copycdaudio_to_cda(CDTOCENT *cdtocent, char *filename, OPTS *opts)
{
char buffer[CDAUDIOREADSIZE];
char overbuf[CDAUDIOREADSIZE];
long starttme;
int overlap;
int bytesread;
int l0;
int l1;
int l2;
int l3;
int ofd;
int curlba;
int framesperread;
#ifdef __NetBSD__
struct audio_info info;
#else
int precision, speed, stereo;
#endif
int audio_fd=-1;

overlap = l0 = l1 = l2 = l3 = 0;

if(G_opts.dostdout == FALSE)
    {
    if((ofd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0600)) < 0)
        {
        fprintf(stderr,"Unable to open output file\n");
        close(F_cdctlfd);
        exit(0);
        }
    }
else
    {
    ofd = fileno(stdout);
    }

starttme = time(NULL);

framesperread = FRAMESPERREAD;

/* We start reading the CD-DA track a bit before where the TOC says it is,
   because in CD-DA mode the CD-ROM is unable to seek to the beginning 
   accurately, and we dont want to miss any bits. */

if(cdtocent->absaddr+AUDIOSTARTOFFSET > 0) 
    {
    curlba = cdtocent->absaddr+AUDIOSTARTOFFSET;
    }
else
    {
    curlba = 0;
    }

if(begin_cdda(F_cdctlfd) != SUCCESS)
    {
    fprintf(stderr,"Unable to enter CDAudio Mode\n");
    close(F_cdctlfd);
    exit(0);
    }

if(G_opts.playaudio == TRUE)
    {
#ifdef __NetBSD__
    if((audio_fd = open ("/dev/audio", O_WRONLY, 0)) < 0)
        {
        fprintf(stderr,"Unable to open audio device\n");
        close(F_cdctlfd);
        exit(0);
        }

    AUDIO_INITINFO(&info);
    info.play.encoding = AUDIO_ENCODING_LINEAR;
    info.play.precision = 16;
    info.play.channels = 2;
    info.play.sample_rate = 44100;
    
    if(ioctl(audio_fd, AUDIO_SETINFO, &info) == -1)
        {
        perror ("configuration of /dev/audio failed");
        close(audio_fd);
        close(F_cdctlfd);
        exit(0);
        }
#else
    if((audio_fd = open ("/dev/dsp", O_WRONLY, 0)) < 0)
        {
        fprintf(stderr,"Unable to open audio device\n");
        close(F_cdctlfd);
        exit(0);
        }

    precision = 16;
    stereo = 1;
    speed = 44100;
    
    if(ioctl(audio_fd, SNDCTL_DSP_SAMPLESIZE, &precision) == -1 ||
          ioctl(audio_fd, SNDCTL_DSP_STEREO, &stereo) == -1 ||
          ioctl(audio_fd, SNDCTL_DSP_SPEED, &speed) == -1)
        {
        perror ("configuration of /dev/dsp failed");
        close(audio_fd);
        close(F_cdctlfd);
        exit(0);
        }
#endif
    }

#ifndef NO_PRIME
    if(cddaread(buffer, curlba, CDDAFRAMESIZE,1, F_cdctlfd) != SUCCESS)
	{
	fprintf(stderr,"\nUnable to prime CD-DA at lba %d\n",curlba);
	goto out;
	}
#endif

for(;
    curlba < cdtocent->absendaddr;
    curlba+=framesperread-READOVERLAP)
    {
    if(framesperread > cdtocent->absendaddr - curlba) 
        {
        framesperread = cdtocent->absendaddr - curlba;

        if(framesperread <= READOVERLAP) break;

        cdd_printf("\n\n    Adjusted framesperread to %d (%d:%d)\n\n", 
                framesperread, cdtocent->absendaddr, curlba);
        }

    bytesread = CDDAFRAMESIZE * framesperread;

    if(cddaread(buffer, curlba, bytesread,framesperread, F_cdctlfd) != SUCCESS)
        {
        fprintf(stderr,"\nUnable to read CD-DA at lba %d\n",curlba);
        break;
        }

    if(READOVERLAP && curlba != cdtocent->absaddr+AUDIOSTARTOFFSET &&
       curlba != 0)
        {
	if(bytesread > CDDAFRAMESIZE)
	    {
	    overlap=CDDAFRAMESIZE;
	    if(memcmp(&overbuf[CDAUDIOREADSIZE-overlap], buffer,
	       overlap) == 0)
		{
		l0++;
		goto overlap;
		}
	}

        if(bytesread > CDDAFRAMESIZE+L1OVERLAPBYTES)
            {
            for(overlap=CDDAFRAMESIZE+L1OVERLAPBYTES; 
                overlap > CDDAFRAMESIZE-L1OVERLAPBYTES; 
                overlap--)
                {
                if(memcmp(&overbuf[CDAUDIOREADSIZE-overlap], buffer, 
                   overlap) == 0) 
                    { 
                    l1++;
                    goto overlap;
                    } 
                }
            }

        if(bytesread > CDDAFRAMESIZE+L2OVERLAPBYTES)
            {
            for(overlap=CDDAFRAMESIZE+L2OVERLAPBYTES; 
                overlap > CDDAFRAMESIZE-L2OVERLAPBYTES; 
                overlap--)
                {
                if(memcmp(&overbuf[CDAUDIOREADSIZE-overlap], buffer, 
                   overlap) == 0) 
                    { 
                    l2++;
                    goto overlap;
                    } 
                }
            }

        for(overlap=bytesread; overlap > MINBYTEOVERLAP; overlap--)
            {
            if(memcmp(&overbuf[CDAUDIOREADSIZE-overlap], buffer, overlap) == 0) 
                { 
                l3++;
                goto overlap;
                } 
            }

        cdd_printf("\n\n    Cant find LBA %d, re-reading..\n\n", curlba);
        curlba-=framesperread+READOVERLAP;
        continue; 
        }

overlap:

    cdd_printf("%d:%d:%d:%d KB/sec: %.2f CLBA: %d Overlap: %d (%d)    %c", 
           l0,l1,l2,l3, (curlba-cdtocent->absaddr)*2.0/(time(NULL)-starttme+1),
           curlba, overlap, overlap-CDDAFRAMESIZE,
	   READOVERLAP&&overlap!=CDDAFRAMESIZE?'\n':'\r');

    fflush(stdout);

    if(write(ofd, &buffer[overlap], bytesread-overlap) != bytesread-overlap)
        {
        perror("Write");
        fprintf(stderr,"\nUnable to write raw CD data\n");
        if(G_opts.dostdout == FALSE) close(ofd);
        close(F_cdctlfd);
        if(G_opts.playaudio == TRUE) close(audio_fd);
        exit(0);
        }

    if(G_opts.playaudio == TRUE)
        {
        if(write(audio_fd, &buffer[overlap], bytesread-overlap) != 
                 bytesread-overlap)
            {
            perror("Audio Write");
            fprintf(stderr,"\nUnable to play CD data\n");
            close(audio_fd);
            if(G_opts.dostdout == FALSE) close(ofd);
            close(F_cdctlfd);
            exit(0);
            }
        }

    /* Copy into buffer, with data as far "right" as possible */

    memcpy(&overbuf[CDAUDIOREADSIZE-bytesread], buffer, bytesread);
    }

out:

if(end_cdda(F_cdctlfd) != SUCCESS)
    {
    fprintf(stderr,"Unable to exit CDAudio Mode\n");
    close(F_cdctlfd);
    exit(0);
    }

if(G_opts.playaudio == TRUE) close(audio_fd);

cdd_printf("\n\n");
if(G_opts.dostdout == FALSE) close(ofd);
}
