 /*****************************************************************************
 * CDD v1.0 Copyright (C) 1996, Charles R. Henrich, All Rights Reserved.      *
 *                                                                            *
 * Use in source and binary forms, with or without modification and           *
 * Redistribution without modification are permitted provided that the        *
 * following conditions are met:                                              *
 *                                                                            *
 * 1. Redistributions of source code must retain the above copyright          *
 *    notice, all internal copyright notices, this list of conditions         *
 *    and the following disclaimer.                                           *
 * 2. Redistributions in binary form must reproduce the above copyright       *
 *    notice, this list of conditions and the following disclaimer in the     *
 *    documentation and/or other materials provided with the distribution.    *
 * 3. All advertising materials mentioning features or use of this software   *
 *    must display the following acknowledgement:                             *
 *      CDD was developed in 1996 by Charles Henrich                          *
 * 4. Neither the name of this product (CDD) nor the name of the author       *
 *    (Charles Henrich) may be used to endorse products that include this     *
 *    product without specific prior written permission.                      *
 * 5. No profit may be made from the inclusion, or distribution of this       *
 *    product.  Although a reasonable service charge for the distribution     *
 *    of this product is permitted where appropriate.                         *
 *                                                                            *
 * THIS SOFTWARE IS PROVIDED BY CHARLES HENRICH `` AS IS '' AND ANY EXPRESS   *
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED          *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE     *
 * DISCLAIMED.  IN NO EVENT SHALL CHARLES HENRICH BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES         *
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR         *
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)         *
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT *
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY  *
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF     *
 * SUCH DAMAGE.                                                               *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Email: henrich@msu.edu                                                     *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <strings.h>
#include <sys/param.h>
#include <scsi.h>
#include "cdd.h"
#include "cdd_cdcmds.h"
#include "cdd_plextorcmds.h"
#include "cdd_toshibacmds.h"
#include "cdd_hpcmds.h"
#include "cdd_neccmds.h"
#include "cdd_panasoniccmds.h"
#include "cdd_mmccmds.h"
#include "cdd_util.h"

int F_pte=0;

DFT F_dtable[]=
    {
        {
        "PLEXTOR CD-ROM PX-4XCS",
        plex_getcdtoc,
        plex_cd_setblocksize,
        plex_cdrawread,
        plex_cddaread,
        plex_begin_cdda,
        plex_end_cdda
        } 
    , 
        {
        "PLEXTOR CD-ROM PX-6XCS",
        plex_getcdtoc,
        plex_cd_setblocksize,
        plex_cdrawread,
        plex_cddaread,
        plex_begin_cdda,
        plex_end_cdda
        } 
    , 
        {
        "PLEXTOR CD-ROM PX-8XCS",
        plex_getcdtoc,
        plex_cd_setblocksize,
        plex_cdrawread,
        plex_cddaread,
        plex_begin_cdda,
        plex_end_cdda
        } 
    , 
        {
        "HP C4324/C4325",
        hp_getcdtoc,
        hp_cd_setblocksize,
        hp_cdrawread,
        hp_cddaread,
        hp_begin_cdda,
        hp_end_cdda
        } 
    , 
        {
        "TOSHIBA CD-ROM XM-5301TA",
        tosh_getcdtoc,
        tosh_cd_setblocksize,
        tosh_cdrawread,
        tosh_cddaread,
        tosh_begin_cdda,
        tosh_end_cdda
        }
    ,
        { 
        "TOSHIBA CD-ROM XM-3701TA",
        tosh_getcdtoc,
        tosh_cd_setblocksize,
        tosh_cdrawread,
        tosh_cddaread,
        tosh_begin_cdda,
        tosh_end_cdda
        }
    ,
        { 
        "NEC CD-ROM DRIVE:501",
        nec_getcdtoc,
        nec_cd_setblocksize,
        nec_cdrawread,
        nec_cddaread,
        nec_begin_cdda,
        nec_end_cdda
        }
    ,
        { 
        "NEC CD-ROM DRIVE:464",
        nec_getcdtoc,
        nec_cd_setblocksize,
        nec_cdrawread,
        nec_cddaread,
        nec_begin_cdda,
        nec_end_cdda
        }
    ,
        { 
        "MATSHITA CD-ROM CR-506",
        panasonic_getcdtoc,
        panasonic_cd_setblocksize,
        panasonic_cdrawread,
        panasonic_cddaread,
        panasonic_begin_cdda,
        panasonic_end_cdda
        }
    ,
        { 
        "SONY CD-ROM CDU311",
        mmc_getcdtoc,
        mmc_cd_setblocksize,
        mmc_cdrawread,
        mmc_cddaread,
        mmc_begin_cdda,
        mmc_end_cdda
        }
    ,
        {
        "END-OF-DEVICES",
        unsupported,
        unsupported,
        unsupported,
        unsupported,
        unsupported,
        unsupported
        }
    };

void doinquiry(int scsifd)
{
char vendor_id[32];
char product_id[32];
char revision[16];
scsireq_t *scsireq;
char buffer[BUFSIZ];
int count;

scsireq=scsireq_new();

if(scsireq == NULL)
    {
    fprintf(stderr,"Out of memory!\n");
    close(scsifd);
    exit(0);
    }

scsireq = scsireq_build(scsireq_new(),
                        96, buffer, SCCMD_READ, "12 0 0 0 v 0", 96);

if (scsireq_enter(scsifd, scsireq) == -1)
    {
    scsi_debug(stderr, -1, scsireq);
    exit(errno);
    }

if(SCSIREQ_ERROR(scsireq) != 0)
    {
    fprintf(stderr,"SCSI Inquiry Failed\n");
    exit(0);
    }

scsireq_decode(scsireq, "s8 z8 z16 z4", vendor_id, product_id, revision);

vendor_id[8] = '\0';
product_id[16] = '\0';
revision[4] = '\0';
    
cdd_printf("Product: %s\n",product_id);
cdd_printf("Vendor: %s\n", vendor_id);
cdd_printf("Revision: %s\n", revision);

for(count=0;
    strcasecmp(F_dtable[count].productid, "END-OF-DEVICES") != 0;
    count++)
    {
    sprintf(buffer,"%s %s", vendor_id, product_id);

    if(strcasecmp(F_dtable[count].productid, buffer) == 0) break;
    }

F_pte = count;

if(strcasecmp(F_dtable[count].productid, "END-OF-DEVICES") == 0)
    {
    cdd_printf("\n");
    cdd_printf(" !!WARNING!! This probably wont work on your CD-ROM!\n\n");

    for(count=0;
        strcasecmp(F_dtable[count].productid, "END-OF-DEVICES") != 0;
        count++)
        {   
        cdd_printf("%d: %s\n", count, F_dtable[count].productid);
        }

    cdd_printf("\n!! WARNING !! Using an incompatable command set ");
    cdd_printf("may cause at the least\n");
    cdd_printf("a system lockup, at the worst you may bit spray your ");
    cdd_printf("SCSI subsystem..  \nYou have been warned!\n\n");
    cdd_printf("Which SCSI command set would you like to try? ");
    cdd_printf("(-1 to quit): ");

    fgets(buffer, BUFSIZ, stdin);

    F_pte = atoi(buffer);
 
    if(F_pte < 0 || F_pte >= count || buffer[0] == '\n') 
        {
        cdd_printf("\n");
        exit(0);
        }
    }

cdd_printf("\n");
}

int cd_setblocksize(int scsifd, int bytessec)
{
return (F_dtable[F_pte].cd_setblocksize_func)(scsifd, bytessec);
}

int getcdtoc(int scsifd, CDTOC *cdtoc)
{
return (F_dtable[F_pte].getcdtoc_func)(scsifd, cdtoc);
}

int cdrawread(char *buffer, int startlba, 
              int blocksize, int blockstoread, int scsifd)
{
return (F_dtable[F_pte].cdrawread_func)
       (buffer, startlba, blocksize, blockstoread, scsifd);
}

int cddaread(char *buffer, int startlba,
             int blocksize, int blockstoread, int scsifd)
{
return (F_dtable[F_pte].cddaread_func)
       (buffer, startlba, blocksize, blockstoread, scsifd);
}

int begin_cdda(int scsifd)
{
return (F_dtable[F_pte].begin_cdda_func)(scsifd);
}

int end_cdda(int scsifd)
{
return (F_dtable[F_pte].end_cdda_func)(scsifd);
}

int unsupported(void)
{
cdd_printf("This operation is not supported on this device!\n");

return FAILURE;
}
