 /*****************************************************************************
 * CDD v1.0 Copyright (C) 1996, Charles R. Henrich, All Rights Reserved.      *
 *                                                                            *
 * Use in source and binary forms, with or without modification and           *
 * Redistribution without modification are permitted provided that the        *
 * following conditions are met:                                              *
 *                                                                            *
 * 1. Redistributions of source code must retain the above copyright          *
 *    notice, all internal copyright notices, this list of conditions         *
 *    and the following disclaimer.                                           *
 * 2. Redistributions in binary form must reproduce the above copyright       *
 *    notice, this list of conditions and the following disclaimer in the     *
 *    documentation and/or other materials provided with the distribution.    *
 * 3. All advertising materials mentioning features or use of this software   *
 *    must display the following acknowledgement:                             *
 *      CDD was developed in 1996 by Charles Henrich                          *
 * 4. Neither the name of this product (CDD) nor the name of the author       *
 *    (Charles Henrich) may be used to endorse products that include this     *
 *    product without specific prior written permission.                      *
 * 5. No profit may be made from the inclusion, or distribution of this       *
 *    product.  Although a reasonable service charge for the distribution     *
 *    of this product is permitted where appropriate.                         *
 *                                                                            *
 * THIS SOFTWARE IS PROVIDED BY CHARLES HENRICH `` AS IS '' AND ANY EXPRESS   *
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED          *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE     *
 * DISCLAIMED.  IN NO EVENT SHALL CHARLES HENRICH BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES         *
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR         *
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)         *
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT *
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY  *
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF     *
 * SUCH DAMAGE.                                                               *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Email: henrich@msu.edu                                                     *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/param.h>
#include <string.h>
#include "cdd.h"
#include "cdd_util.h"
#include "st.h"

#define NETBUFSIZ 256;
#define CDDB_SERVER "cddb.sonic.net"
#define CDDB_PORT 888
#define CDDB_TIMEOUT 30

int getcddbtoc(CDTOC *toc)
{
SOCK *cddbserver;
char *argv[10];
char hostname[MAXHOSTNAMELEN];
char catagory[BUFSIZ];
int retcode;
int count;
int ret;
int track;
long discid;

for(count=0;count<toc->numentries;count++)
    {
    toc->tocents[count].trackname[0] = '\0';
    }

if(gethostname(hostname, MAXHOSTNAMELEN) == -1) strcpy(hostname, "UNKNOWN");

cddbserver = st_connect(CDDB_SERVER, CDDB_PORT);

if(cddbserver == NULL) return FAILURE;

if(st_read(cddbserver, CDDB_TIMEOUT) != SUCCESS)
    {
    st_close(cddbserver);
    return FAILURE;
    }

ret=sscanf(cddbserver->buffer,"%d", &retcode);

if(ret != 1 || retcode != 201) 
    {
    st_close(cddbserver);
    return FAILURE;
    }

st_write(cddbserver, "cddb hello %s %s CDD %s\r\n", getenv("USER"), hostname,
         VERSION);

if(st_read(cddbserver, CDDB_TIMEOUT) != SUCCESS)
    {
    st_close(cddbserver);
    return FAILURE;
    }

ret=sscanf(cddbserver->buffer,"%d", &retcode);

if(ret != 1 || retcode != 200)
    {
    st_close(cddbserver);
    return FAILURE;
    }

discid = cddb_discid(toc);

st_write(cddbserver, "cddb query %08x %d ", discid, toc->numentries-1);

for(count=0;count<toc->numentries;count++)
    {
    if(toc->tocents[count].tracknumber >= 0xA0) 
        {
        st_write(cddbserver, "%d\r\n", (toc->tocents[count].absaddr+150) / 75);
        break;
        }

    st_write(cddbserver, "%d ", toc->tocents[count].absaddr+150);
    }

if(st_read(cddbserver, CDDB_TIMEOUT) != SUCCESS)
    {
    st_close(cddbserver);
    return FAILURE;
    }

ret=sscanf(cddbserver->buffer,"%d %s %*s %[^\n]", 
           &retcode, catagory, toc->cdname);

if(ret != 3 || (retcode != 200 && retcode != 211))
    {
    st_close(cddbserver);
    return FAILURE;
    }

if(retcode == 211)
    {
    while(st_read(cddbserver, CDDB_TIMEOUT) == SUCCESS)
        {
        if(cddbserver->buffer[0] == '.') break;
        
        sscanf(cddbserver->buffer,"%s %x %[^\n]", catagory, &discid, 
               toc->cdname);

        cdd_printf("Multiple Matches: %s", cddbserver->buffer);
        }

    cdd_printf("\n");
    }


for(count=0;toc->cdname[count];count++)
    {
    if(toc->cdname[count] == '/') toc->cdname[count] = '-'; 

    if(toc->cdname[count] == '\r' || toc->cdname[count] == '\n') 
        {
        toc->cdname[count] = '\0';
        }
    }

strcat(G_opts.workdir, "/");
strcat(G_opts.workdir, toc->cdname);

st_write(cddbserver, "cddb read %s %08x\r\n", catagory, discid);

if(st_read(cddbserver, CDDB_TIMEOUT) != SUCCESS)
    {
    st_close(cddbserver);
    return FAILURE;
    }

ret=sscanf(cddbserver->buffer,"%d", &retcode);

if(ret != 1 || retcode != 210)
    {
    st_close(cddbserver);
    return FAILURE;
    }

track = 0;

while(st_read(cddbserver, CDDB_TIMEOUT) == SUCCESS)
    {
    if(cddbserver->buffer[0] == '.') break;

    if(strncmp(cddbserver->buffer, "TTITLE", 6) != 0) continue;

    ret=split(cddbserver->buffer, argv, '=');
  
    if(ret != 2) continue;

    for(count=0;argv[1][count];count++)
        {
        if(argv[1][count] == '/') argv[1][count] = '-'; 

        if(argv[1][count] == '\r' || argv[1][count] == '\n') 
            {
            argv[1][count] = '\0';
            }
        }

    strncpy(toc->tocents[track].trackname, argv[1], BUFSIZ-1);

    toc->tocents[track].trackname[BUFSIZ-1] = '\0';

    track++;
    }

st_write(cddbserver, "quit\r\n");
st_close(cddbserver);

return SUCCESS;
}
