 /*****************************************************************************
 * CDD v1.0 Copyright (C) 1996, Charles R. Henrich, All Rights Reserved.      *
 *                                                                            *
 * Use in source and binary forms, with or without modification and           *
 * Redistribution without modification are permitted provided that the        *
 * following conditions are met:                                              *
 *                                                                            *
 * 1. Redistributions of source code must retain the above copyright          *
 *    notice, all internal copyright notices, this list of conditions         *
 *    and the following disclaimer.                                           *
 * 2. Redistributions in binary form must reproduce the above copyright       *
 *    notice, this list of conditions and the following disclaimer in the     *
 *    documentation and/or other materials provided with the distribution.    *
 * 3. All advertising materials mentioning features or use of this software   *
 *    must display the following acknowledgement:                             *
 *      CDD was developed in 1996 by Charles Henrich                          *
 * 4. Neither the name of this product (CDD) nor the name of the author       *
 *    (Charles Henrich) may be used to endorse products that include this     *
 *    product without specific prior written permission.                      *
 * 5. No profit may be made from the inclusion, or distribution of this       *
 *    product.  Although a reasonable service charge for the distribution     *
 *    of this product is permitted where appropriate.                         *
 *                                                                            *
 * THIS SOFTWARE IS PROVIDED BY CHARLES HENRICH `` AS IS '' AND ANY EXPRESS   *
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED          *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE     *
 * DISCLAIMED.  IN NO EVENT SHALL CHARLES HENRICH BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES         *
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR         *
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)         *
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT *
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY  *
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF     *
 * SUCH DAMAGE.                                                               *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Email: henrich@msu.edu                                                     *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <unistd.h>
#include <scsi.h>
#include "cdd.h"
#include "cdd_cdcmds.h"
#include "cdd_neccmds.h"
#include "cdd_util.h"

scsireq_t F_scsireq;

int nec_cd_setblocksize(int scsifd, int bytessec)
{
return SUCCESS;
}

int nec_getcdtoc(int scsifd, CDTOC *cdtoc)
{
unsigned char *tocptr;
unsigned char toc[MAXTOCSIZE];
int count;

if(scsireq_reset(&F_scsireq) == NULL)
    {
    cdd_printf("Unable to initialize SCSI request\n");
    return FAILURE;
    }

if(scsireq_build(&F_scsireq, MAXTOCSIZE, toc, SCCMD_READ,
                 "43 0 0 0 0 0 0 i2 i1", MAXTOCSIZE, 0 ) == NULL)
    {
    cdd_printf("Unable to build SCSI request\n");
    return FAILURE;
    }

if(scsireq_enter(scsifd, &F_scsireq) == -1)
    {
    cdd_printf("Unable to send SCSI request\n");
    scsi_debug(stderr, -1, &F_scsireq);
    return FAILURE;
    }

if(SCSIREQ_ERROR((&F_scsireq)) != 0)
    {
    cdd_printf("SCSI Command failed: Read TOC\n");
    return FAILURE;
    }

scsireq_decode(&F_scsireq, "i2 i1 i1",
               &(cdtoc->toclen), &(cdtoc->firsttrack), &(cdtoc->lasttrack));

cdtoc->numentries = (cdtoc->toclen - 2)/TOCENTRYSIZE;

if(cdtoc->numentries <= 0)
    {
    cdd_printf("Scrambled TOC!\n");
    return FAILURE;
    }

for(count=0,tocptr = &toc[4];
    count < cdtoc->numentries;
    tocptr += TOCENTRYSIZE, count++)
    {
    scsireq_buff_decode(tocptr, TOCENTRYSIZE, "s1 *b4 b1 b1 b1 b1 i1 s+1 i4",
                        &cdtoc->tocents[count].nchan,
                        &cdtoc->tocents[count].ttype,
                        &cdtoc->tocents[count].dcp,
                        &cdtoc->tocents[count].preemph,
                        &cdtoc->tocents[count].tracknumber,
                        &cdtoc->tocents[count].absaddr);

    if(count > 0)
        {
        /* The end of the previous track is the start of this track minus some
           space.  It is really supposed to be 150 frames from the start of
           this track, but because we cannot seek very accurately in CD-DA mode
           and we dont want to lose the end of the CD-DA data, we just go ahead
           and read the silence in the track-change gap */

        cdtoc->tocents[count-1].absendaddr =
                                cdtoc->tocents[count].absaddr+AUDIOSTARTOFFSET;
        }
    else
        {
        cdtoc->tocents[count].absendaddr = -1;
        }

    if(cdtoc->tocents[count].tracknumber >= 0xA0) break;
    }

return SUCCESS;
}

int nec_cdrawread(char *buffer, int startlba,
                   int blocksize, int blockstoread, int scsifd)
{
if(scsireq_reset(&F_scsireq) == NULL)
    {
    cdd_printf("Unable to initialize SCSI request\n");
    return FAILURE;
    }

if(scsireq_build(&F_scsireq, blocksize, buffer, SCCMD_READ,
                 "a4 0 i4 i3 0", startlba, blockstoread)
                 != &F_scsireq)
    {
    cdd_printf("Unable to build SCSI READ request\n");
    return FAILURE;
    }

if(scsireq_enter(scsifd, &F_scsireq) == -1)
    {
    cdd_printf("Unable to send SCSI READ request\n");
    scsi_debug(stderr, -1, &F_scsireq);
    return FAILURE;
    }

if(SCSIREQ_ERROR((&F_scsireq)) != 0)
    {
    cdd_printf("\nSCSI Command failed on lba %d\n",startlba);
    return FAILURE;
    }

return SUCCESS;
}

int nec_cddaread(char *buffer, int startlba,
                   int blocksize, int blockstoread, int scsifd)
{
if(scsireq_reset(&F_scsireq) == NULL)
    {
    cdd_printf("Unable to initialize SCSI request\n");
    return FAILURE;
    }

if(scsireq_build(&F_scsireq, blocksize, buffer, SCCMD_READ,
                 "d4 0 i4 i3 0", startlba, blockstoread)
                 != &F_scsireq)
    {
    cdd_printf("Unable to build SCSI READ request\n");
    return FAILURE;
    }

if(scsireq_enter(scsifd, &F_scsireq) == -1)
    {
    cdd_printf("Unable to send SCSI READ request\n");
    scsi_debug(stderr, -1, &F_scsireq);
    return FAILURE;
    }

if(SCSIREQ_ERROR((&F_scsireq)) != 0)
    {
    cdd_printf("\nSCSI Command failed on lba %d\n",startlba);
    return FAILURE;
    }

return SUCCESS;
}

int nec_begin_cdda(int scsifd)
{
return SUCCESS;
}

int nec_end_cdda(int scsifd)
{
return SUCCESS;
}
