#include <stdio.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include "cdd.h"

/*      
 *   cddb - CD Database Management Library
 *
 *   Copyright (C) 1993-1996  Ti Kan
 *   E-mail: ti@amb.org
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *      
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License  
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *      
 */     

/*      
 * cddb_sum
 *      Convert an integer to its text string representation, and
 *      compute its checksum.  Used by cddb_discid to derive the
 *      disc ID.
 *
 * Args:
 *      n - The integer value.
 *
 * Return:      
 *      The integer checksum.
 */     

static unsigned long
cddb_sum(unsigned long n)
{
        char    buf[12],
                *p; 
        int     ret = 0;

        /* For backward compatibility this algorithm must not change */
        sprintf(buf, "%lu", n);
        for (p = buf; *p != '\0'; p++)
                ret += (*p - '0');
                
        return (ret);
}       

/*      
 * cddb_discid
 *      Compute a magic disc ID based on the number of tracks,
 *      the length of each track, and a checksum of the string
 *      that represents the offset of each track.
 *
 * Args:
 *      s - Pointer to the curstat_t structure.
 *
 * Return:      
 *      The integer disc ID.
 */     

/* Routine modified by Charles Henrich (henrich@msu.edu) for cdd */

unsigned long
cddb_discid(CDTOC *toc)
{
        int     i,
                t = 0,
                n = 0;

        /* For backward compatibility this algorithm must not change */

        for (i = 0; i < toc->numentries-1; i++) {

                n += cddb_sum((toc->tocents[i].absaddr+150)/75);

                t += ((toc->tocents[i+1].absaddr+150)/75) - 
                     ((toc->tocents[i].absaddr+150)/75);
        }

#ifdef XMCD
        /* For backward compatibility this algorithm must not change */
        for (i = 0; i < (int) s->tot_trks; i++) {
                n += cddb_sum((s->trkinfo[i].min * 60) + s->trkinfo[i].sec);

                t += ((s->trkinfo[i+1].min * 60) + s->trkinfo[i+1].sec) -
                     ((s->trkinfo[i].min * 60) + s->trkinfo[i].sec);
        }
#endif

        return ((n % 0xff) << 24 | (t << 8) | (toc->numentries-1));
}                       
