/****************************************************************************/
/*                                                                          */
/* These routines were written by Charles Henrich                           */
/*                                                                          */
/* Copyright (c) 1993 Michigan State University Board of Trustee's          */
/*                                                                          */
/****************************************************************************/

#ifndef _ST_H_
#define _ST_H_

#include <sys/param.h>

#define LOCAL_SOCKET_ERROR          -1
#define BIND_ERROR                  -2
#define LISTEN_ERROR                -3
#define SELECT_ERROR                -4
#define NUMBER_OF_DESCRIPTORS_ERROR -5
#define RESET_DESCRIPTORS_ERROR     -6
#define INVALID_CALLBACK            -7
#define PARTIAL_STRING              -8
#define UNKNOWN_HOSTNAME            -9
#define CANT_GET_SOCKET             -10
#define INVALID_SOCK                -11
#define TIMEOUT                     -12
#define ITERATIONS_COMPLETE         -13
#define SOCKOPT_ERROR               -14
#define DES_SET_KEY_FAILURE         -15

#define SUCCESS                      1
#define TRUE                         1
#define FAILURE                     -1
#define FALSE                       -1
#define MASSIVE_FAILURE             -2

#define st_NEW_CONNECTION            0
#define st_CLOSE_CONNECTION          1
#define st_READ                      2
#define st_WRITE                     3
#define st_INPUT_TIMEOUT             4
#define st_PRE_SELECT                5

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 256
#endif

 /**************************************/
 /*                                    */
 /* Public info-sharing structure def. */
 /*                                    */
 /**************************************/

typedef struct callback_structure
    {
    char buffer[BUFSIZ];
    char hostname[MAXHOSTNAMELEN];
    int callback;
    int descriptor;
    } SOCK;

 /**************************************/
 /*                                    */
 /* Public function declarations       */
 /*                                    */
 /**************************************/

int st_mainloop(int iterations);
int st_init(char *passwd);
int st_remove_callback(int callback);
int st_add_callback(int callback, void function(SOCK *));
int st_reset_callbacks(void);
int st_set_timeout(int timeout);
SOCK *st_connect(char *host, int port);
int st_write(SOCK *entry, char *fmt, ...);
int st_read(SOCK *entry, int timeout);
void st_close(SOCK *entry);
char *st_error(int error);
int st_listen(int port); 

#endif /* _ST_H_ */
